package com.book.push.service.wx.impl;

import com.book.push.config.RedisProperties;
import com.book.push.dao.pojo.Platform;
import com.book.push.handler.*;
import com.book.push.service.dao.PlatformService;
import com.book.push.service.wx.WxThirdPartService;
import lombok.AllArgsConstructor;
import me.chanjar.weixin.open.api.WxOpenService;
import me.chanjar.weixin.open.api.impl.WxOpenInRedisConfigStorage;
import me.chanjar.weixin.open.api.impl.WxOpenServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.stereotype.Service;
import redis.clients.jedis.JedisPool;

import javax.annotation.PostConstruct;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import static me.chanjar.weixin.common.api.WxConsts.EventType;
import static me.chanjar.weixin.common.api.WxConsts.EventType.SUBSCRIBE;
import static me.chanjar.weixin.common.api.WxConsts.EventType.UNSUBSCRIBE;
import static me.chanjar.weixin.common.api.WxConsts.XmlMsgType;
import static me.chanjar.weixin.common.api.WxConsts.XmlMsgType.EVENT;
import static me.chanjar.weixin.mp.constant.WxMpEventConstants.CustomerService.*;
import static me.chanjar.weixin.mp.constant.WxMpEventConstants.POI_CHECK_NOTIFY;

/**
 * created in 2021/8/20
 * Project: book-store
 *
 * @author win7
 */
@Service
@AllArgsConstructor
@EnableConfigurationProperties(RedisProperties.class)
public class WxThirdPartServiceImpl implements WxThirdPartService {
    private Logger logger = LoggerFactory.getLogger(getClass());
    private final LogHandler logHandler;
    private final NullHandler nullHandler;
    private final KfSessionHandler kfSessionHandler;
    private final StoreCheckNotifyHandler storeCheckNotifyHandler;
    private final LocationHandler locationHandler;
    private final MenuHandler menuHandler;
    private final MsgHandler msgHandler;
    private final UnsubscribeHandler unsubscribeHandler;
    private final SubscribeHandler subscribeHandler;
    private final ScanHandler scanHandler;
    private volatile ConcurrentHashMap<Integer, WxOpenService> wxOpenServiceMap = new ConcurrentHashMap();
    @Autowired
    private RedisProperties redisProperties;
    @Autowired
    private PlatformService platformService;
    @Autowired
    private JedisPool jedisPool;

    private WxThirdPartMessageRouter wxThirdPartMessageRouter;
public static final String KEY_PREFIX = "platformId_%d" ;
    @PostConstruct
    public void load() {
        Map<Integer, Platform> integerPlatformMap = platformService.selectAll();
        Set<Integer> platFormIdSet = integerPlatformMap.keySet();
        ConcurrentHashMap newWxOpenServiceMap = new ConcurrentHashMap<Integer, WxOpenService>();
        platFormIdSet.stream().forEach(id -> {
            Platform platform = integerPlatformMap.get(id);
            WxOpenService wxOpenService = new WxOpenServiceImpl();
            WxOpenInRedisConfigStorage inRedisConfigStorage = new WxOpenInRedisConfigStorage(jedisPool,String.format(KEY_PREFIX, id));
            inRedisConfigStorage.setComponentAppId(platform.getAppid());
            inRedisConfigStorage.setComponentAppSecret(platform.getSecret());

            //FIXME  这里有错误，请注意修改
            inRedisConfigStorage.setComponentToken(platform.getToken());//错误位置
            inRedisConfigStorage.setComponentAesKey(platform.getAesKey());
            wxOpenService.setWxOpenConfigStorage(inRedisConfigStorage);
            wxOpenServiceMap.put(id, wxOpenService);
        });
        wxOpenServiceMap = newWxOpenServiceMap;

        wxThirdPartMessageRouter = new WxThirdPartMessageRouter(this);
        // 记录所有事件的日志 （异步执行）
        wxThirdPartMessageRouter.rule().handler(this.logHandler).next();

        // 接收客服会话管理事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(KF_CREATE_SESSION)
                .handler(this.kfSessionHandler).end();
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(KF_CLOSE_SESSION)
                .handler(this.kfSessionHandler).end();
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(KF_SWITCH_SESSION)
                .handler(this.kfSessionHandler).end();

        // 门店审核事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(POI_CHECK_NOTIFY).handler(this.storeCheckNotifyHandler).end();

        // 自定义菜单事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(EventType.CLICK).handler(this.menuHandler).end();

        // 点击菜单连接事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(EventType.VIEW).handler(this.nullHandler).end();

        // 关注事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(SUBSCRIBE).handler(this.subscribeHandler).end();

        // 取消关注事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(UNSUBSCRIBE).handler(this.unsubscribeHandler).end();

        // 上报地理位置事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(EventType.LOCATION).handler(this.locationHandler).end();

        // 接收地理位置消息
        wxThirdPartMessageRouter.rule().async(false).msgType(XmlMsgType.LOCATION).handler(this.locationHandler).end();

        // 扫码事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(EventType.SCAN).handler(this.scanHandler).end();

        // 默认
        wxThirdPartMessageRouter.rule().async(false).handler(this.msgHandler).end();


    }

    @Override
    public WxOpenService getWxOpenServiceByPlatFormId(Integer platFormId) {
        return wxOpenServiceMap.get(platFormId);
    }
}
