package com.book.server.service.impl;


import com.book.server.common.util.JsonUtils;
import com.book.server.dao.entity.*;
import com.book.server.dao.entity.example.*;
import com.book.server.dao.mapper.*;
import com.book.server.model.VO.*;
import com.book.server.service.BookService;
import com.book.server.utils.HttpTool;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class BookServiceImpl implements BookService {
    public static final Logger logger = LoggerFactory.getLogger(BookServiceImpl.class);
    @Autowired
    private BookMapper bookMapper;
    @Autowired
    private BookMapper2 bookMapper2;

    @Autowired
    private ManageBlockMapper manageBlockMapper;

    @Autowired
    private ManageBlockResourceMapper manageBlockResourceMapper;
    @Autowired
    private BookCategoryMapper bookCategoryMapper;
    @Autowired
    private SearchKeywordMapper searchKeywordMapper;
    @Autowired
    private ManageBlockResource2Mapper manageBlockResource2Mapper;

    public List<BookCategory> getAllCategory() {
        List<BookCategory> bookCategories = bookCategoryMapper.selectByExample(
                BookCategoryExample.newAndCreateCriteria().example()
        );
        return bookCategories;
    }


    @Override
    public List<BookRes> query(QueryVO queryVO) {
        queryVO.setQuery("%" + queryVO.getQuery() + "%");
        List<BookRes> bookResList = bookMapper2.queryBookRes(queryVO, queryVO.getPage() * queryVO.getSize(), queryVO.getSize());
        return bookResList;
    }

    @Override
    public List<BookRes> recommendByRead(QueryVO queryVO) {
        ArrayList<BookRes> bookRes = new ArrayList<>();
        if (queryVO.getPage() == 5) {
            return bookRes;
        }
        List<BookRes> books = bookMapper2.novelEndRecommend(queryVO,
                queryVO.getPage() * queryVO.getSize(), queryVO.getSize());
        return books;
    }

    @Override
    public List<BlockRes> getBlockByPageId(Integer pageId) {
        ArrayList<BlockRes> list = new ArrayList<>();
        // 获取板块
        ManageBlockExample manageBlockExample = ManageBlockExample.newAndCreateCriteria()
                .andPageIdEqualTo(pageId)
                .example();
        manageBlockExample.orderBy(ManageBlock.Column.weigh.desc());
        List<ManageBlock> manageBlocks = manageBlockMapper.selectByExample(manageBlockExample);


        // 获取已有版本的所有书籍
        List<Integer> blockIds = manageBlocks.stream().map(ManageBlock::getId).collect(Collectors.toList());
        List<ManageBlockResourceRes> blockResourceResList = manageBlockResource2Mapper.getBooks(blockIds);

        // 合并相同 block 书籍
        HashMap<Integer, TreeSet<ManageBlockResourceRes>> map = new HashMap<>();
        blockResourceResList.forEach(x -> {
            TreeSet<ManageBlockResourceRes> set = map.computeIfAbsent(x.getBlockId(), k -> new TreeSet<>((o1, o2) -> o2.getWeigh() - o1.getWeigh()));
            set.add(x);
        });

        manageBlocks.forEach(x -> {
            BlockRes blockRes = new BlockRes();
            blockRes.setName(x.getName());
            blockRes.setType(x.getType());
            blockRes.setSecondName(x.getSecondName());
            blockRes.setResources(map.getOrDefault(x.getId(), new TreeSet<>()));
            list.add(blockRes);
        });
        return list;
    }

    @Override
    public void insertBooks() {

        //插入书籍基本信息
        String bookInfoUrl = "http://new.emeixs.com/index.php/Home/Interface/listing";

        String jsonStr = HttpTool.sendGet(bookInfoUrl, "");
        JsonObject jsonObject = JsonUtils.getObject(jsonStr, JsonObject.class);
        JsonArray list = jsonObject.getAsJsonArray("list");

        Map<String, String> typeMap = new HashMap<String, String>();
        typeMap.put("15", "豪门总裁");
        typeMap.put("16", "婚恋家庭");
        typeMap.put("17", "豪门虐恋");
        typeMap.put("18", "甜宠萌宝");
        typeMap.put("19", "重生逆袭");
        typeMap.put("20", "穿越重生");
        typeMap.put("21", "权谋宫斗");
        typeMap.put("22", "经商种田");
        typeMap.put("23", "出版");
        typeMap.put("24", "其他");
        List<Book> bookList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            JsonElement jsonElement = list.get(i);
            JsonObject bookObject = jsonElement.getAsJsonObject();
            String author_name = bookObject.get("author_name").getAsString();
            String book_brief = bookObject.get("book_brief").getAsString();
            String book_id = bookObject.get("book_id").getAsString();
            String book_name = bookObject.get("book_name").getAsString();
            String chapter = bookObject.get("chapter").getAsString();
            String charge = bookObject.get("charge").getAsString();
            String cover = bookObject.get("cover").getAsString();
            String ctype_id = bookObject.get("ctype_id").getAsString();
            String gender = bookObject.get("gender").getAsString();
            String is_show = bookObject.get("is_show").getAsString();
            String state = bookObject.get("state").getAsString();
            String type_id = bookObject.get("type_id").getAsString();
            String words = bookObject.get("words").getAsString();

            //章节列表
            String chaptersStr = HttpTool.sendGet("http://new.emeixs.com/index.php/Home/Interface/chapters/bid/" + book_id, "");
            JsonParser parser = new JsonParser();
            JsonArray jsonArray = parser.parse(chaptersStr).getAsJsonArray();
            JsonObject firstChapters = jsonArray.get(1).getAsJsonObject();
            String first_chapter_id = firstChapters.get("content_id").getAsString();
            String first_chapter_name = firstChapters.get("name").getAsString();
            JsonObject lastChapters = jsonArray.get(jsonArray.size() - 1).getAsJsonObject();
            String last_chapter_id = lastChapters.get("content_id").getAsString();
            String last_chapter_name = lastChapters.get("name").getAsString();


            Book book = new Book();
            book.setId(Long.parseLong(book_id));
            book.setBookCategoryId(Integer.parseInt(type_id));
            book.setName(book_name);
            book.setRealname(book_name);
            book.setAuthor(author_name);
            book.setImage(cover);
            book.setDescription(book_brief);
            book.setSex(gender);
            book.setBillingType(String.valueOf(Integer.parseInt(charge) + 1));
            book.setWordCount(words);
            book.setChapterNum(Integer.valueOf(chapter));
//            book.setLastChapterUtime(Integer.parseInt(System.currentTimeMillis()+""));
            book.setCreatetime(Integer.parseInt(System.currentTimeMillis() / 1000 + ""));
            book.setCpId(Integer.parseInt(book_id));
            book.setCpName(book_name);
            book.setTags(typeMap.get(ctype_id));
            book.setState("1");
            book.setPrice(45);
            book.setArticleChapterOrder(4);
            book.setAttentChapterOrder(4);
            book.setCornerMark("hot");
            book.setIsFinish(String.valueOf(Integer.parseInt(state) - 1));
            book.setReferralNum(new Byte((byte) 5));
            book.setPutAdSet(new Byte((byte) 1));
            book.setClassifyWhiteList(new Byte((byte) 0));
            book.setCheckRank(new Byte((byte) 0));
            book.setRank(new Byte((byte) 6));
            book.setExpireTime(Integer.MAX_VALUE);
            book.setLastChapterUtime(Integer.parseInt(System.currentTimeMillis() / 1000 + ""));
            book.setAppPrice(45);
            book.setFirstChapterId(Long.parseLong(first_chapter_id));
            book.setFirstChapterName(first_chapter_name);
            book.setLastChapterId(Long.parseLong(last_chapter_id));
            book.setLastChapterName(last_chapter_name);
            book.setCansee("1");
            book.setRank(new Byte((byte) 6));

            bookList.add(book);


        }

        for (Book book : bookList) {
            int i = new Random().nextInt(bookList.size());
            book.setRecommandBookId(bookList.get(i).getId());
            System.out.println(JsonUtils.toJsonStr(book));
            bookMapper.insert(book);


        }

    }

    @Override
    public List<Chapter> getChaptersByBookId(String bookId) {
        List<Chapter> list = new ArrayList<>();
        try {
            if (StringUtils.isEmpty(bookId)) {
                throw new IllegalArgumentException("bookId cannot be null or empty");
            }
            final String chaptersUrl = "http://new.emeixs.com/index.php/Home/Interface/chapters/bid/" + bookId;
            String result = HttpTool.sendGet(chaptersUrl, "");
            if (StringUtils.isEmpty(result)) {
                return list;
            }
            JsonParser parser = new JsonParser();
            JsonArray jsonArray = parser.parse(result).getAsJsonArray();

            for (int i = 0; i < jsonArray.size(); i++) {
                JsonObject jsonObject = jsonArray.get(i).getAsJsonObject();
                Chapter chapter = new Chapter();
                chapter.setBookId(jsonObject.get("book_id").getAsString());
                chapter.setContentId(jsonObject.get("content_id").getAsString());
                chapter.setIsPay(jsonObject.get("ispay").getAsString());
                chapter.setName(jsonObject.get("name").getAsString());
                chapter.setNum(jsonObject.get("num").getAsString());
                chapter.setType(jsonObject.get("type").getAsString());
                chapter.setVolumeId(jsonObject.get("volume_id").getAsString());
                chapter.setWords(jsonObject.get("words").getAsString());
                list.add(chapter);
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
            e.printStackTrace();

            return list;
        }

        return list;
    }

    @Override
    public List<SearchKeyword> getSearchKeywordBySex(String sex) {
        SearchKeywordExample example = SearchKeywordExample.newAndCreateCriteria()
                .andSexEqualTo(sex)
                .example();
        example.orderBy(SearchKeyword.Column.weigh.desc());
        List<SearchKeyword> searchKeywords = searchKeywordMapper.selectByExample(example);
        return searchKeywords;
    }

    @Override
    public List<BookRes> topRecommend(QueryVO queryVO) {
        List<BookRes> bookRes = bookMapper2.topRecommend(queryVO, 0, 20);
        return getRandom(bookRes);
    }

    @Override
    public List<BookRes> smartRecommand(QueryVO queryVO) {
        List<BookRes> bookRes = bookMapper2.smartRecommand(queryVO);
        return bookRes;
    }

    private List<BookRes> getRandom(List<BookRes> bookRes) {
        if (bookRes.size() <= 4) {
            return bookRes;
        }
        Collections.shuffle(bookRes);
        return bookRes.subList(0, 4);
    }


}
